;;; update-cdb.el --- Update and/or setup character attribute database

;; Copyright (C) 2002 MORIOKA Tomohiko.

;; Author: MORIOKA Tomohiko <tomo@kanji.zinbun.kyoto-u.ac.jp>
;; Keywords: Character, Database, UTF-2000, Unicode, UCS-4, MULE.

;; This file is part of XEmacs UTF-2000.

;; XEmacs UTF-2000 is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2, or (at
;; your option) any later version.

;; XEmacs UTF-2000 is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with XEmacs UTF-2000; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(defun delete-file-with-children (filename)
  (if (file-directory-p filename)
      (let ((files
	     (directory-files filename 'full "^[^.]" 'so-sort)))
	(if files
	    (dolist (file files)
	      (delete-file-with-children file)))
	(remove-directory filename))
    (delete-file filename)))

(cond
 ((featurep 'chise)
  (defvar system-char-database-directory
    (expand-file-name "char-db"
		      (or exec-directory
			  "../lib-src/")))

  (defun file-name-char-attribute-name (filename)
    (let ((i 0)
	  (base 0)
	  (len (length filename))
	  chr dest)
      (while (< i len)
	(if (eq (setq chr (aref filename i)) ?%)
	    (setq dest (concat dest
			       (substring filename base i)
			       (char-to-string
				(int-char
				 (string-to-int
				  (substring filename (1+ i) (+ i 3)) 16))))
		  i (+ i 3)
		  base i)
	  (setq i (1+ i))))
      (concat dest (substring filename base len))))

  (cond
   ((or load-ignore-elc-files
	(not (file-exists-p system-char-database-directory)))
    (if (file-exists-p system-char-database-directory)
	(delete-file-with-children system-char-database-directory))

    (load "dumped-chars.el")
    (dolist (file system-char-db-source-file-list)
      (pureload file))

    (dolist (attribute (char-attribute-list))
      (save-char-attribute-table attribute))

    (dolist (ccs (charset-list))
      (save-charset-mapping-table ccs))
    )
   (t
    (mapcar (lambda (file)
	      (mount-char-attribute-table
	       (intern (file-name-char-attribute-name file))))
	    (directory-files
	     (expand-file-name "system-char-id"
			       system-char-database-directory)
	     nil nil t t))
    )))
 (t
  (load "dumped-chars.el")
  (dolist (file system-char-db-source-file-list)
    (pureload file))
  ))

(garbage-collect)

;;; update-cdb.el ends here
